package upp

import (
	"crypto/tls"
	"log"

	"github.com/elastic/go-elasticsearch/v7"
	"github.com/loveuer/upp/pkg/api"
	"github.com/loveuer/upp/pkg/cache"
	"github.com/loveuer/upp/pkg/db"
	"gorm.io/gorm"
)

type module func(u *upp)

func InitDB(uri string, models ...any) module {
	db, err := db.New(uri)
	if err != nil {
		log.Panic(err.Error())
	}

	if err = db.AutoMigrate(models...); err != nil {
		log.Panic(err.Error())
	}

	return func(u *upp) {
		u.db = db
	}
}

func (u *upp) UseDB() *gorm.DB {
	tx := u.db.Session(&gorm.Session{})
	if u.debug {
		tx = tx.Debug()
	}
	return tx
}

func InitCache(uri string) module {
	cache, err := cache.New(uri)
	if err != nil {
		log.Panic(err.Error())
	}

	return func(u *upp) {
		u.cache = cache
	}
}

func (u *upp) UseCache() cache.Cache {
	return u.cache
}

func (u *upp) UseES() *elasticsearch.Client {
	return nil
}

type ApiConfig struct {
	Address   string
	TLSConfig *tls.Config
}

func InitApi(api *api.App, cfgs ...ApiConfig) module {
	cfg := ApiConfig{}
	if len(cfgs) > 0 {
		cfg = cfgs[0]
	}

	if cfg.Address == "" {
		cfg.Address = "localhost:8080"
	}

	return func(u *upp) {
		api.Upp = u
		u.api = &uppApi{
			engine: api,
			config: cfg,
		}
	}
}
