package upp

import (
	"context"
	"os/signal"
	"syscall"

	"github.com/loveuer/upp/pkg/api"
	"github.com/loveuer/upp/pkg/cache"
	"github.com/loveuer/upp/pkg/interfaces"
	"github.com/loveuer/upp/pkg/log"
	"github.com/loveuer/upp/pkg/tool"
	"gorm.io/gorm"
)

type uppApi struct {
	engine *api.App
	config ApiConfig
}

type upp struct {
	debug  bool
	logger interfaces.Logger
	db     *gorm.DB
	cache  cache.Cache
	api    *uppApi
}

func (u *upp) With(modules ...module) {
	for _, m := range modules {
		m(u)
	}
}

func (u *upp) Run(ctx context.Context) {
	u.StartAPI(ctx)

	<-ctx.Done()
}

func (u *upp) RunSignal() {
	ctx, cancel := signal.NotifyContext(context.Background(), syscall.SIGHUP, syscall.SIGINT, syscall.SIGTERM, syscall.SIGQUIT)
	defer cancel()

	if u.api != nil {
		u.StartAPI(ctx)
	}

	<-ctx.Done()

	u.Warn(" UPP | quit by signal...")

	<-tool.Timeout(2).Done()
}

func (u *upp) StartAPI(ctx context.Context) {
	u.Info("UPP | run api at %s", u.api.config.Address)
	go u.api.engine.Run(u.api.config.Address)
	go func() {
		<-ctx.Done()
		u.api.engine.Shutdown(tool.Timeout(2))
	}()
}

func New() *upp {
	app := &upp{
		logger: log.New(),
	}

	return app
}
